#!./perl
#************************************************************************
#myri_configure.pl - configure a system to boot across the ATOMIC network
#1.0
# 
#Copyright (c) 1995 University of Southern California.
#All rights reserved.                                            
#                                                                
#Redistribution and use in source and binary forms are permitted
#provided that the above copyright notice and this paragraph are
#duplicated in all such forms and that any documentation, advertising
#materials, and other materials related to such distribution and use
#acknowledge that the software was developed by the University of
#Southern California, Information Sciences Institute.  The name of the
#University may not be used to endorse or promote products derived from
#this software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
#WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
#MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#************************************************************************

@INC = ( @INC, "." );

require "request.pl";
require "cp.pl";

sub newfile {
# create a new file with the contents in $contents, and create a backup as
# well.  $file is the new file and $backup is the backup file.

	local($file,$backup,$contents) = @_;

	if ( -f $backup ) {
		unlink $backup 
			if &y_or_n("$backup exists, delete it?","y",STDERR);
	}

		

	print STDERR (( -f $backup ) ? 
		"Modifying $file.  No new backup made\n" :
		"Modifying $file.  Backup is in $backup.\n" );
	rename($file, $backup) if -f $to && $backup && !-f $backup;
	open(TO,">$file") || die "Can't open $file for writing : $!\n";
	print TO "$contents\n";
	close TO;
}

!$> || die "You must be root to run $0\n";

if ( @ARGV < 1 || @ARGV > 5 ) {
	die <<"END";
Usage $0 unix router domain hostname_file device

	unix is the suffix of a file with the root /vmunix. that is to be
		the kernel loaded when the machine is rebooted.
	router is the name of the default router to use.  If you are
		planning to boot this kernel from the myrinet, the machine
		must be able to reach NFS servers via this router.
	domain is the NIS domain for this machine.  There must be an NIS
		server that maps that domain on the attached myrinet if this
		server is going to boot standalone.
	hostname_file is the name of the file that contains the name of
		this host when booting standalone from the myrinet.  The
		file should be of the form hostname.device and the name it
		contains must be in /etc/hosts (not just the NIS hosts map).
	device is the major device number of the /dev/myri* and /dev/mlanai*
		device files.

	All these parameters except unix are optional
END
}
else { ($unix, $router, $domain, $hostname_file, $device) = @ARGV; }

chop ($arch=`arch -k`);

if ( &check("/etc/rc.boot","myri" ) ) {
	print STDERR "\n/etc/rc.boot recognizes Myricom (no changes made)\n";
}
else {
	$to="/etc/rc.boot";
	$from="/etc/rc.boot.atomic.hold";
	print STDERR "\nchanging rc.boot to recognize the myrinet\n";
	rename($to,$from) || die "Can't back up $to\n";
	print STDERR "You may have to remove the line \"ifconfig -ad auto-revarp up\" from rc.boot\n" if $arch eq "sun4m";

	open(FROM,$from) || die "Can't open $from\n";
	open(TO,">$to") || die "Can't open $to\n";
	while ( <FROM> ) {

		# bug in ISI's rc.boot

		s'route -f add default `cat /etc/defaultrouter` 1'route -f add default `shcat /etc/defaultrouter` 1';

		print TO $_;
		if ( (/# Set hostname/../^fi/) =~/E0/ ) {
			print TO <<"END";
 
#Added to check for hostname.myri0 if there's no hostname.le0 -- tvf
if [ -z "$hostname" ]; then
       hostname="`shcat /etc/hostname.????0 2>/dev/null`"
       hostname $hostname
fi
END
		}
	}
	close TO ; close FROM;
	print STDERR "$to modified.  Original is in $from.\n";
}

if (!(-f "/sbin/route" && -x "/sbin/route")) {
	$where = "./route-static";
	$where = &get_string("Enter the filename of the staticly linked route command\nor a null string to skip this ","","/$^/ || -r","No such file") 
		if ! -r $where;
	if ( $where ) {
		&cp($where, "/sbin/route");
		chmod(0755,"/sbin/route") || 
			warn "Check permissions on /sbin/route\n";
		print STDERR "\nCopied $where to /sbin\n\n";
	}
	else { print "route command not copied\n\n"; }
}
else { print STDERR "\nA staticly linked route command exists in /sbin\n"; }

print STDERR "Linking /vmunix.$unix to /vmunix\n";

if ( -f "/vmunix" ) {
	($t1,$t2,$t3,$links) = stat("/vmunix");
	unlink "/vmunix" if $links > 1 || 
		&y_or_n("/vmunix is not a link.  Delete it?","n");
}

if ( !-f "/vmunix" ) {
	if ( !-f "/vmunix.$unix" ) {
		$where=&get_string("/vmunix.$unix doesn't exist.  Where should I copy it from? ","","-f","No such file",STDERR);
		&cp("$where", "/vmunix.$unix");
		chmod(0755,"/vmunix.$unix") || 
			warn "Check permissions on /vmunix.$unix\n";
		print STDERR "Copied $where to /vmunix.$unix\n";
	}

	link("/vmunix.$unix","/vmunix") || 
		die "Can't make link to /vmunix.$unix :$!\n";
}
else { warn "/vmunix not changed.\n";}

print "\n";
&newfile("/etc/defaultrouter","/etc/defaultrouter.atomic.hold",$router) 
	if $router;

print "\n";
&newfile("/etc/defaultdomain","/etc/defaultdomain.atomic.hold",$domain) 
	if $domain;

STEP:
{ if ( $hostname_file ) {
	chdir "/etc";
	if ( !-f $hostname_file && !-f "not.$hostname_file" ) {
		$host_contents = &get_string("No /etc/$hostname_file exists.\nPlease enter the name that the host will use from this interface","",1,STDERR);
		if ( ! $host_contents ) {
			print STDERR "Aborting hostname change\n";
			next STEP;
		}
		open(FILE,">$hostname_file") || 
			die "Can't open $hostname_file :$!\n";
		print FILE "$host_contents\n";
		close FILE;
	}
	opendir(ETC,"/etc") || die "\nCan't read /etc : $!\n";
	@files = grep(/^hostname.*/,readdir(ETC));
	closedir(ETC);
	for (@files) {
		rename($_, "not.$_") || 
			die "\nError renaming $_ : $!\n/etc may be a mess\n";
	}
	rename("not.$hostname_file",$hostname_file) ||
		die "\nError renaming $hostname_file back : $!\n";
	print STDERR "\nhostname file is now $hostname_file\n";
} }

print "\n";
if ($device ) {
	for ("/dev/myri0","/dev/mlanai0") {
		-e && -c && unlink;
		(system "mknod",$_,"c",$device,"0") && 
			warn "Can't mknod $_ : $!\n" && next;
		-e && -c && 
			(chmod 0666, $_ || warn "Check permissions of $_ : $!\n");
		print "$_ created\n";	
	} 
}
