#!./perl
#************************************************************************
#myri_kernel.pl - patch kernel source to include myricom driver
#1.0
# 
#Copyright (c) 1995 University of Southern California.
#All rights reserved.                                            
#                                                                
#Redistribution and use in source and binary forms are permitted
#provided that the above copyright notice and this paragraph are
#duplicated in all such forms and that any documentation, advertising
#materials, and other materials related to such distribution and use
#acknowledge that the software was developed by the University of
#Southern California, Information Sciences Institute.  The name of the
#University may not be used to endorse or promote products derived from
#this software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
#WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
#MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#************************************************************************
# Parameters are the base kernel configuration name and new kernel
# configuration name 

@INC = ( @INC, "." );

require "request.pl";
require "cp.pl";

sub change_conf {
#This was formerly an awk script, but should work in this incarnation, too.

	local($conf) = @_;	# The new filename
	local($backup);		# Where we put the original file
	local($in_switch);	# true if we're parsing lines in the
				# declaration of cdevsw
	local($record);		# The device number of the current entry
	local($comma);		# The last character of the previous line.
				# If this *is* a comma when it's time to
				# output the myricom entry, we don't need to
				# print one.

	$backup = "$conf.atomic.hold";

	print STDERR <<"END";
Changing $conf.
Original file is in $backup.

END
	rename($conf,$backup) || die "Can't rename $conf\n";
	$conf = ">$conf";
	open(F,$backup) || die "Can't open $backup\n";
	open(CONF,">$conf") || die "Can't open $conf for writing\n";
	while ( <F> ) {

		# insert the declarations and defines for the myrinet 
		# interface.  seltrue is just a convenient naem for which
		# to look 

		/extern int seltrue\(\);/ && do {
			print CONF <<"END";

#include "myri.h"
#if NMYRI > 0
extern myri_open();
extern myri_close();
extern myri_ioctl();
extern myri_select();
extern myri_mmap();
#else
#define myri_open nodev
#define myri_close nodev
#define myri_ioctl nodev
#define myri_select nodev
#define myri_mmap nodev
#endif

END
		};

		#detect the character device table

		/struct.*cdevsw\[\]/ && do {
			$in_switch = 1;
			$record = 0;
		};

		#count the entries in the character device table

		(/}[^;]/ || /}$/) && do {
			if ( $in_switch ) {
				/(.*)\s*/;
				$comma = substr($1,-1,1);
				$record ++;
			}
		};

		# this is the end of the table.  Insert the myrinet driver info

		/};/  && do {
			if ( $in_switch ) {
				print CONF "," unless $comma eq ",";
				print CONF <<"END";
    {
	myri_open,	myri_close,	nulldev,	nulldev,	/*$record*/
	myri_ioctl,	nulldev,	myri_select,	myri_mmap,
	0, 0
    },
END
				$in_switch = 0;
			}
		};
		print CONF $_;
	}
	close F; close CONF;
	$record;
}

$arch=`arch -k` || "sun";
chop $arch if substr($arch,-1,1) eq "\n";

$kpath="/usr/sys";
$myripath="$ENV{MYRI_HOME}/install";

print STDERR <<"END";
This script modifies files in your kernel source directories
(specifically sun/conf.c and the conf/files for the architecture you
specify) and builds a kernel source directory that will contain source
for a kernel that includes a myricom driver.  Backups of the source
files will be made.  If you don't want to do this hit control-c now.
END

$kpath = &get_string("Where is the source directory you want to modify",$kpath,"-d && -r && -x","Bad directory - check spelling & permissions",STDERR);
$myripath = &get_string("Where is the directory containing the Myrinet driver
objects",$myripath,"-d && -r && -x","Bad directory - check spelling & permissions",STDERR);
$arch=&get_string("What is the architecture of your machine",$arch, "-r \"$myripath/myri.\$_.multi.o\"","Bad architecture - no driver exists",STDERR);
$driver= &y_or_n("Use multicast driver?","y",SDTERR) ? 
	"$myripath/myri.$arch.multi.o":
	"$myripath/myri.$arch.o";

if (&check("$kpath/sun/conf.c","MYRI") ) {
	print STDERR <<"END";

$kpath/sun/conf.c already has been modified to include the driver
(no changes made)

END
}
else { 
	$devno = &change_conf("$kpath/sun/conf.c");
	print STDERR <<"END";

The myricom device will have major number $devno.  Be sure the /dev/myri0 and 
/dev/mlanai0 have a major number of $devno when running this kernel.

END
}


$replace = 1;
$driverdest="$kpath/$arch/OBJ/myri.o";

print STDERR "Copying device driver to $driverdest\n";

$replace = &y_or_n("$driverdest exists.  Replace it?","y") if -e $driverdest;

if ( $replace ) {
	&cp($driver,$driverdest);
	chmod(0644,$driverdest) || warn "Check permissions on $driverdest\n";
	print STDERR "Device driver copied\n\n";
}
else { print "\n"; }

$conf="$kpath/$arch/conf/files";
$backup="$kpath/$arch/conf/files.atomic.hold";

if (&check($conf,"myri")) {
	print STDERR <<"END";
$conf already has been modified (no changes made)

END
}
else { 
	rename($conf,$backup) || die "Can't rename $conf\n";
	open(CONF,">$conf") || die "Can't open $conf for write: $!\n";
	open(OLD,$backup) || die "Can't open $backup : $!\n";
	while (<OLD>) { print CONF $_;}
	print CONF <<"END";
/usr/atom/atomic/myri_net/src/driver/myri.c 	optional 	myri 	device-driver
END
	close CONF; close OLD;
	print STDERR "$conf updated.  Old file is in $backup.\n\n";
}

chdir "$kpath/$arch/conf/";	# So we can use get_string easily

$conf = &get_string("What is the name of the configuration on which to base the old kernel","GENERIC","-r",STDERR);
$new = &get_string("What is the name of the new kernel configuration","IPATOM",1,STDERR);

if ( !-e $new || &y_or_n("Configuration $new exists: overwrite?","n")){
	open(OLD,"$conf") || die "Can't open $conf : $!\n";
	open(NEW,">$new") || die "Can't open $new for write: $!\n";
	while (<OLD> ) {
		s/\"$conf\"/\"$new\"/;
		s/$conf\./$new./;
		print NEW $_;
	}
	print NEW <<"END";
device-driver	myri			# Myricom Atomic LAN driver
END
	close NEW; close OLD;
}
else { print "Existing configuartion $new unchanged\n\n"; }

if ( &y_or_n("\nConfigure kernel?","y") ) {
	print "Configuring....\n";
	system "config",$new;
	die "config $new failed : $!\n" unless $? == 0;
	print STDERR "Kernel configured\n\n";
}
else {exit 0;}

if ( &y_or_n("Build kernel?","y") ) {
	chdir "$kpath/$arch/$new";
	system "make";
	die "make failed : $!\n" unless $? == 0;
	print STDERR "Kernel built\n\n";
}
else {exit 0;}
print "kernel is in $kpath/$arch/$new/vmunix \n";

# if the script is run as root, we offer the option of creating the
# device special files

if ( $> == 1 && &y_or_n("Create device files?","y") ) {
	for ("/dev/myri0","/dev/mlanai0") {
		-e && -c && unlink;
		(system "mknod",$_,"c",$devno,"0") && warn "Can't mknod $_ : $!\n";
		-e && -c && (chmod 0666, $_ || warn "Check permissions of $_ : $!\n");
	}
}
else { print "use device $devno\n" if defined($devno); }
